@@ -1,11 +1,6 @@
 #ifndef __INHERITED_XS_COMPAT_H_
 #define __INHERITED_XS_COMPAT_H_
 
-#if (PERL_VERSION < 21)
-#undef mg_findext
-#define NEED_mg_findext
-#endif
-
 #if (PERL_VERSION >= 12)
 #define CAIX_OPTIMIZE_OPMETHOD
 #define CAIX_OPTIMIZE_OPMETHOD_RESULT (&PL_sv_yes)
@@ -22,6 +17,27 @@
 #define dNOOP
 #endif
 
+#if defined(GvGPFLAGS_on) || defined(GvGPFLAGS_off)
+#error("GvGPFLAGS_on or GvGPFLAGS_off defined by perl core");
+#endif
+
+#ifndef GvGPFLAGS
+#define GvGPFLAGS(gv) (GvLINE(gv) & ((U32)1<<31))
+#define GvGPFLAGS_on(gv) (GvLINE(gv) |= ((U32)1<<31))
+#define GvGPFLAGS_off(gv) (GvLINE(gv) &= ~((U32)1<<31))
+#else
+#define GvGPFLAGS_on(gv) (GvGPFLAGS(gv) = 1)
+#define GvGPFLAGS_off(gv) (GvGPFLAGS(gv) = 0)
+#endif
+
+#ifndef HvENAME
+#define HvENAME HvNAME
+#define HvENAME_HEK HvNAME_HEK
+#endif
+
+#define hv_fetchhek(hv, hek) \
+    ((SV **)hv_common((hv), NULL, HEK_KEY(hek), HEK_LEN(hek), HEK_UTF8(hek), HV_FETCH_JUST_SV, NULL, HEK_HASH(hek)))
+
 #ifndef gv_init_sv
 #define gv_init_sv(gv, stash, sv, flags) gv_init(gv, stash, SvPVX(sv), SvLEN(sv), flags | SvUTF8(sv))
 #endif
